@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, Red Hat.
+ * Copyright (c) 2012-2013, Red Hat.
  * Copyright (c) 2006, Ken McDonell.  All Rights Reserved.
  * Copyright (c) 2006-2009, Aconex.  All Rights Reserved.
  * 
@@ -50,6 +50,10 @@ PmChart::PmChart() : QMainWindow(NULL)
 
     my.statusBar = new StatusBar;
     setStatusBar(my.statusBar);
+    connect(my.statusBar->timeFrame(), SIGNAL(clicked()),
+				this, SLOT(editSamples()));
+    connect(my.statusBar->timeButton(), SIGNAL(clicked()),
+				this, SLOT(optionsShowTimeControl()));
 
     my.timeAxisRightAlign = toolBar->height();
     toolBar->setAllowedAreas(Qt::RightToolBarArea | Qt::TopToolBarArea);
@@ -60,9 +64,6 @@ PmChart::PmChart() : QMainWindow(NULL)
     if (!globalSettings.initialToolbar && !outfile)
 	toolBar->hide();
 
-    my.liveHidden = true;
-    my.archiveHidden = true;
-    timeControlAction->setChecked(false);
     toolbarAction->setChecked(true);
     my.toolbarHidden = !globalSettings.initialToolbar;
     my.consoleHidden = true;
@@ -104,15 +105,10 @@ void PmChart::setupDialogs(void)
     my.samples = new SamplesDialog(this);
     my.exporter = new ExportDialog(this);
     my.newchart = new ChartDialog(this);
-    my.editchart = new ChartDialog(this);
     my.openview = new OpenViewDialog(this);
     my.saveview = new SaveViewDialog(this);
     my.settings = new SettingsDialog(this);
 
-    connect(my.statusBar->timeFrame(), SIGNAL(clicked()),
-				this, SLOT(editSamples()));
-    connect(my.statusBar->timeButton(), SIGNAL(clicked()),
-				this, SLOT(optionsTimeControl()));
     connect(my.newtab->buttonOk, SIGNAL(clicked()),
 				this, SLOT(acceptNewTab()));
     connect(my.edittab->buttonOk, SIGNAL(clicked()),
@@ -129,19 +125,14 @@ void PmChart::quit()
     // End any processes we may have started and close any open dialogs
     if (my.dialogsSetup) {
 	my.info->reject();
-	my.search->reject();
-	my.newtab->reject();
-	my.edittab->reject();
-	my.samples->reject();
-	my.exporter->reject();
-	my.newchart->reject();
-	my.editchart->reject();
-	my.openview->reject();
-	my.saveview->reject();
-	my.settings->reject();
     }
     if (pmtime)
 	pmtime->quit();
+#ifdef HAVE_UNSETENV
+    unsetenv("PCP_STDERR");
+#else
+    putenv("PCP_STDERR=");
+#endif
     pmflush();
 }
 
@@ -206,9 +197,11 @@ void PmChart::updateHeight(int adjustment)
     int ideal = newSize.height() + adjustment;	// may be negative
     int sized = QApplication::desktop()->availableGeometry().height();
 
+#ifdef DESPERATE
     console->post(PmChart::DebugUi,
-	"upateHeight() oldsize h=%d,w=%d maximum=%d proposed=%d",
+	"updateHeight() oldsize h=%d,w=%d maximum=%d proposed=%d",
 	newSize.height(), newSize.width(), sized, ideal);
+#endif
 
     if (ideal > sized)
 	ideal = sized;
@@ -393,7 +386,6 @@ void PmChart::helpManual()
     bool ok;
     QString documents("file://" HTMLDIR);
     QString separator = QString(__pmPathSeparator());
-    documents.append(separator).append("html");
     documents.append(separator).append("index.html");
     ok = QDesktopServices::openUrl(QUrl(documents, QUrl::TolerantMode));
     if (!ok) {
@@ -424,24 +416,20 @@ void PmChart::whatsThis()
     QWhatsThis::enterWhatsThisMode();
 }
 
-void PmChart::optionsTimeControl()
+void PmChart::optionsShowTimeControl()
 {
-    if (activeTab()->isArchiveSource()) {
-	if (my.archiveHidden)
-	    pmtime->showArchiveTimeControl();
-	else
-	    pmtime->hideArchiveTimeControl();
-	my.archiveHidden = !my.archiveHidden;
-	timeControlAction->setChecked(!my.archiveHidden);
-    }
-    else {
-	if (my.liveHidden)
-	    pmtime->showLiveTimeControl();
-	else
-	    pmtime->hideLiveTimeControl();
-	my.liveHidden = !my.liveHidden;
-	timeControlAction->setChecked(!my.liveHidden);
-    }
+    if (activeTab()->isArchiveSource())
+	pmtime->showArchiveTimeControl();
+    else
+	pmtime->showLiveTimeControl();
+}
+
+void PmChart::optionsHideTimeControl()
+{
+    if (activeTab()->isArchiveSource())
+	pmtime->hideArchiveTimeControl();
+    else
+	pmtime->hideLiveTimeControl();
 }
 
 void PmChart::optionsToolbar()
@@ -485,19 +473,12 @@ void PmChart::optionsNewPmchart()
     buddy->start("pmchart", arguments);
 }
 
-void PmChart::createNewChart()
-{
-    setupDialogs();
-    my.newchart->reset();
-    my.newchart->show();
-}
-
 //
-// Note: Called from both Accept and OK on New Chart dialog
+// Note: Called from both Apply and OK on New Chart dialog
 // Must only called when the changes are definately going to
 // be applied, so all input validation must be done by now.
 //
-void PmChart::acceptNewChart()
+Chart *PmChart::acceptNewChart()
 {
     bool yAutoScale;
     double yMin, yMax;
@@ -525,11 +506,14 @@ void PmChart::acceptNewChart()
     activeTab()->showGadgets();
 
     enableUi();
+    return cp;
 }
 
 void PmChart::fileNewChart()
 {
-    createNewChart();
+    setupDialogs();
+    my.newchart->reset();
+    my.newchart->show();
 }
 
 void PmChart::editChart()
@@ -537,8 +521,8 @@ void PmChart::editChart()
     Chart *cp = (Chart *)activeTab()->currentGadget();
 
     setupDialogs();
-    my.editchart->reset(cp);
-    my.editchart->show();
+    my.newchart->reset(cp);
+    my.newchart->show();
 }
 
 void PmChart::acceptEditChart()
@@ -548,17 +532,17 @@ void PmChart::acceptEditChart()
     QString scheme;
     int sequence;
 
-    Chart *cp = my.editchart->chart();
-    QString editTitle = my.editchart->title().trimmed();
+    Chart *cp = my.newchart->chart();
+    QString editTitle = my.newchart->title().trimmed();
     if (editTitle.isEmpty() == false && editTitle != cp->title())
 	cp->changeTitle(editTitle, true);
-    cp->setLegendVisible(my.editchart->legend());
-    cp->setAntiAliasing(my.editchart->antiAliasing());
-    cp->setRateConvert(my.editchart->rateConvert());
-    my.editchart->scale(&yAutoScale, &yMin, &yMax);
+    cp->setLegendVisible(my.newchart->legend());
+    cp->setAntiAliasing(my.newchart->antiAliasing());
+    cp->setRateConvert(my.newchart->rateConvert());
+    my.newchart->scale(&yAutoScale, &yMin, &yMax);
     cp->setScale(yAutoScale, yMin, yMax);
-    my.editchart->setupChartPlots(cp);
-    my.editchart->scheme(&scheme, &sequence);
+    my.newchart->setupChartPlots(cp);
+    my.newchart->scheme(&scheme, &sequence);
     cp->setScheme(scheme, sequence);
     if (cp->metricCount() > 0) {
 	cp->replot();
@@ -592,9 +576,11 @@ void PmChart::metricSearch(QTreeWidget *pmns)
 
 void PmChart::editSamples()
 {
+    int samples = activeGroup->sampleHistory();
+    int visible = activeGroup->visibleHistory();
+
     setupDialogs();
-    my.samples->reset(globalSettings.sampleHistory,
-			globalSettings.visibleHistory);
+    my.samples->reset(samples, visible);
     my.samples->show();
 }
 
@@ -711,13 +697,10 @@ void PmChart::setActiveTab(int index, bool redisplay)
 {
     console->post("PmChart::setActiveTab index=%d r=%d", index, redisplay);
     
-    if (chartTabWidget->setActiveTab(index) == true) {
+    if (chartTabWidget->setActiveTab(index) == true)
 	activeGroup = archiveGroup;
-	timeControlAction->setChecked(!my.archiveHidden);
-    } else {
+    else
 	activeGroup = liveGroup;
-	timeControlAction->setChecked(!my.liveHidden);
-    }
     activeGroup->updateTimeButton();
     activeGroup->updateTimeAxis();
 
@@ -822,7 +805,8 @@ void PmChart::setupEnabledActionsList()
     addSeparatorAction();	// end recording group
     my.toolbarActionsList << editSettingsAction;
     addSeparatorAction();	// end settings group
-    my.toolbarActionsList << timeControlAction << newPmchartAction;
+    my.toolbarActionsList << showTimeControlAction << hideTimeControlAction
+			  << newPmchartAction;
     addSeparatorAction();	// end other processes
     my.toolbarActionsList << helpManualAction << helpWhatsThisAction;
 
@@ -833,7 +817,7 @@ void PmChart::setupEnabledActionsList()
 				// separator
 			  << fileExportAction
 				// separator
-			  << timeControlAction << newPmchartAction;
+			  << showTimeControlAction << newPmchartAction;
 
     if (globalSettings.toolbarActions.size() > 0) {
 	setEnabledActionsList(globalSettings.toolbarActions, false);
@@ -894,8 +878,6 @@ void PmChart::newScheme(QString cs)
 {
     my.newchart->setCurrentScheme(cs);
     my.newchart->setupSchemeComboBox();
-    my.editchart->setCurrentScheme(cs);
-    my.editchart->setupSchemeComboBox();
 }
 
 void PmChart::exportFile()
