@@ -49,6 +49,7 @@ public:
     QString endTime() { return timeString(&my.end); }
     QString desc() const { return my.desc; }
     char *descAscii() const { return strdup((const char *)my.desc.toAscii()); }
+    QString context_hostname() const { return my.context_hostname; }
 
     // Number of active contexts to this source
     uint numContexts() const { return my.handles.size(); }
@@ -75,10 +76,6 @@ public:
     static QString timeString(const struct timeval *timeval);
     static QString timeStringBrief(const struct timeval *timeval);
 
-    void setSecureFlag() { my.flags |= PM_CTXFLAG_SECURE; }
-    void setRelaxedFlag() { my.flags |= PM_CTXFLAG_RELAXED; }
-    void setCompressFlag() { my.flags |= PM_CTXFLAG_COMPRESS; }
-
 protected:
     // retry context/connection (e.g. if it failed in the constructor)
     void retryConnect(int type, QString &source);
@@ -92,7 +89,9 @@ private:
 	int type;
 	QString source;
 	QString proxy;
+	QString attrs;
 	QString host;
+	QString context_hostname; // from pmcd/archive, not from -h/-a argument
 	QString	desc;
 	QString timezone;
 	QList<int> handles;	// Contexts created for this source
